"use strict";

exports.__esModule = true;
exports.default = useHoverEvents;

var React = _interopRequireWildcard(require("react"));

var _modality = require("../__modules__/modality");

var _dispatchCustomEvent = _interopRequireDefault(require("../__modules__/dispatchCustomEvent"));

var _supportsPointerEvent = _interopRequireDefault(require("../__modules__/supportsPointerEvent"));

var _useEvent = _interopRequireDefault(require("../use-event"));

var _useLayoutEffect = _interopRequireDefault(require("../__modules__/useLayoutEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/**
 * Implementation
 */
var opts = {
  passive: true
}; // This accounts for the non-PointerEvent fallback events.

function getPointerType(event) {
  var pointerType = event.pointerType;
  return pointerType != null ? pointerType : (0, _modality.getModality)();
}

function useHoverEvents(targetRef, config) {
  var contain = config.contain,
      disabled = config.disabled,
      onHoverStart = config.onHoverStart,
      onHoverChange = config.onHoverChange,
      onHoverUpdate = config.onHoverUpdate,
      onHoverEnd = config.onHoverEnd;
  var canUsePE = (0, _supportsPointerEvent.default)();
  var moveHandle = (0, _useEvent.default)(canUsePE ? 'pointermove' : 'mousemove', opts);
  var enterHandle = (0, _useEvent.default)(canUsePE ? 'pointerenter' : 'mouseenter', opts);
  var leaveHandle = (0, _useEvent.default)(canUsePE ? 'pointerleave' : 'mouseleave', opts); // These custom events are used to implement the "contain" prop.

  var lockHandle = (0, _useEvent.default)('react-gui:hover:lock', opts);
  var unlockHandle = (0, _useEvent.default)('react-gui:hover:unlock', opts);
  /**
   * Hover end gesture
   */

  var hoverEnd = React.useCallback(function (e, target) {
    if (onHoverEnd != null) {
      onHoverEnd(e);
    }

    if (onHoverChange != null) {
      onHoverChange(false);
    } // Remove the listeners once finished.


    moveHandle.setListener(target, null);
    leaveHandle.setListener(target, null);
  }, [onHoverChange, onHoverEnd, moveHandle, leaveHandle]);
  /**
   * Leave element
   */

  var leaveListener = React.useCallback(function (e) {
    var target = targetRef.current;

    if (target != null && getPointerType(e) !== 'touch') {
      if (contain) {
        (0, _dispatchCustomEvent.default)(target, 'react-gui:hover:unlock');
      }

      hoverEnd(e, target);
    }
  }, [contain, hoverEnd, targetRef]);
  /**
   * Move within element
   */

  var moveListener = React.useCallback(function (e) {
    if (getPointerType(e) !== 'touch') {
      if (onHoverUpdate != null) {
        // Not all browsers have these properties
        if (e.x == null) {
          e.x = e.clientX;
        }

        if (e.y == null) {
          e.y = e.clientY;
        }

        onHoverUpdate(e);
      }
    }
  }, [onHoverUpdate]);
  /**
   * Hover start gesture
   */

  var hoverStart = React.useCallback(function (e, target) {
    if (onHoverStart != null) {
      onHoverStart(e);
    }

    if (onHoverChange != null) {
      onHoverChange(true);
    } // Set the listeners needed for the rest of the hover gesture.


    if (onHoverUpdate != null) {
      moveHandle.setListener(target, !disabled ? moveListener : null);
    }

    leaveHandle.setListener(target, !disabled ? leaveListener : null);
  }, [disabled, onHoverStart, onHoverChange, onHoverUpdate, moveHandle, moveListener, leaveHandle, leaveListener]);
  /**
   * Enter element
   */

  var enterListener = React.useCallback(function (e) {
    var target = targetRef.current;

    if (target != null && getPointerType(e) !== 'touch') {
      if (contain) {
        (0, _dispatchCustomEvent.default)(target, 'react-gui:hover:lock');
      }

      hoverStart(e, target);

      var lockListener = function lockListener(lockEvent) {
        if (lockEvent.target !== target) {
          hoverEnd(e, target);
        }
      };

      var unlockListener = function unlockListener(lockEvent) {
        if (lockEvent.target !== target) {
          hoverStart(e, target);
        }
      };

      lockHandle.setListener(target, !disabled ? lockListener : null);
      unlockHandle.setListener(target, !disabled ? unlockListener : null);
    }
  }, [contain, disabled, hoverStart, hoverEnd, lockHandle, unlockHandle, targetRef]);
  (0, _useLayoutEffect.default)(function () {
    var target = targetRef.current;

    if (target !== null) {
      enterHandle.setListener(target, !disabled ? enterListener : null);
    }
  }, [disabled, enterHandle, enterListener, targetRef]);
}

module.exports = exports.default;